"""
@author: Junguang Jiang
@contact: JiangJunguang1123@outlook.com
"""

import os
import random
import numpy as np


class Jnu:
    """Jnu Dataset.

    dataset link: https://pan.xunlei.com/s/VNNVrpOoaEAPPdIODQaJQQcQA1?pwd=jnri# extract code: jnri

    Args:
        root (str): Root directory of dataset
        rpm (str): The task (domain) to create dataset. Choices include ``'600'``, \
            ``'800'`` and ``'1000'``.
        length (int): window length (or num of vibrations of each input)
        hop_size (int): hop length (or num of vibrations of each input
        shuffle (bool): whether shuffle dataset
    """
    domain_list = {
        "600": ['n600.csv', 'ib600.csv', 'ob600.csv', 'tb600.csv'],
        "800": ['n800.csv', 'ib800.csv', 'ob800.csv', 'tb800.csv'],
        "1000": ['n1000.csv', 'ib1000.csv', 'ob1000.csv', 'tb1000.csv']
    }
    CLASSES = ['Normal Condition', 'Inner Ball', 'Outer Fault', 'Ball Fault']

    def __init__(self, root: str, rpm: str, length=1024, hop_size=1024, shuffle=False) -> None:
        assert rpm in self.domain_list.keys()
        super(Jnu, self).__init__()
        self.x, self.y = self._get_domain_data(root, rpm, length, hop_size)

        if shuffle:
            self._shuffle()
        self.classes = self.CLASSES
        self.num_classes = len(self.classes)

    def _get_domain_data(self, root, rpm, length, hop_size) -> tuple[np.ndarray, np.ndarray]:
        files = [os.path.join(root, file) for file in self.domain_list[rpm]]
        samples, targets = [], []

        for index, file in enumerate(files):
            sample, target = self._get_file_data(file, index, length, hop_size)
            samples.append(sample), targets.append(target)
        return np.vstack(samples), np.concatenate(targets, axis=0)

    def _get_file_data(self, file, label, length, hop_size) -> tuple[np.ndarray, np.ndarray]:
        samples = []
        data = np.loadtxt(file)
        start_index = 0
        while(start_index + length < data.size):
            samples.append(data[start_index: start_index + length])
            start_index += hop_size
        return np.vstack(samples).astype(np.float32), np.full(len(samples), label, dtype=np.longlong)

    def _shuffle(self):
        # shuffle training samples
        index = list(range(self.x.shape[0]))
        random.Random(0).shuffle(index)
        self.x = self.x[index]
        self.y = self.y[index]



import torch
import random
import numbers
from torch import Tensor
from typing import Sequence
import torchvision
from torch.nn import functional as F
from torch.nn.functional import grid_sample, conv2d, conv1d, interpolate, pad as torch_pad
import warnings
import torchaudio.transforms as T

warnings.filterwarnings("ignore", category=DeprecationWarning)

__all__ = [
    'Compose',
    "Normalization",

    "HShift",
    "VShift",
    "HFlip",
    "VFlip",

    "RandomTimeMasking",
    "TimeMasking",
    "GaussianNoise",
    "RandomScaling",

    "Rotate",
]


class Compose(torchvision.transforms.Compose):
    def __init__(self, transforms):
        super(Compose, self).__init__(transforms)


class Normalization(torch.nn.Module):
    def __int__(self) -> None:
        super(Normalization, self).__int__()

    @torch.no_grad()
    def forward(self, x: Tensor) -> Tensor:
        _min, _ = torch.min(x, -1)
        _max, _ = torch.max(x, -1)
        _min, _max = _min.unsqueeze(-1), _max.unsqueeze(-1)
        return -1. + 2 * (x - _min) / (_max - _min)


class RandomScaling(torch.nn.Module):
    def __init__(self, p=0.5) -> None:
        super(RandomScaling, self).__init__()
        self.p = p

    @torch.no_grad()
    def forward(self, x: Tensor) -> Tensor:
        x = x.clone()
        '''
        scaling the amplify of signal
        '''
        b = x.shape[0]
        random_seed = torch.rand(b, device=x.device) < self.p
        scale_ratio = torch.rand(b, device=x.device) + 0.5
        scaled_x = []
        for index in range(b):
            scaled_x.append(x[index] * scale_ratio[index] if random_seed[index] else x[index])
        return torch.stack(scaled_x)


def make_mask(x: Tensor, start: int, end: int) -> Tensor:
    mask = torch.ones_like(x, device=x.device)
    if start <= end:
        mask[..., start: end] = 0.
    else:
        mask[..., :end] = mask[..., start:] = 0.
    return mask


class RandomTimeMasking(torch.nn.Module):
    """
    Single sample augment
    Random Crop consequent vector of length of size from original vector,  Cropped areas padded with zero
    Args:
        snr (float): all pixels equal or above this value are inverted.
        p (float): probability of the image being color inverted. Default value is 0.5
    """

    def __init__(self, min_time=100, max_time=500, p=0.25) -> None:
        super(RandomTimeMasking, self).__init__()
        self.min_time = min_time
        self.max_time = max_time
        self.p = p

    @torch.no_grad()
    def forward(self, x: Tensor) -> Tensor:
        x = x.clone()
        """
        Args:
            x:
        Returns:
            PIL Image or Tensor: Randomly solarized image.
        """
        b, w = x.shape[0], x.shape[-1]
        random_seed = torch.rand(b) < self.p
        masked_x = []
        for index in range(b):
            if not random_seed[index]:
                masked_x.append(x[index])
            else:
                time_mask = random.randint(self.min_time, self.max_time)
                time_start = random.randint(0, w)
                mask = make_mask(x[index], time_start, (time_start + time_mask) % w)
                masked_x.append(mask * x[index])
        return torch.stack(masked_x)


class GaussianNoise(torch.nn.Module):
    """
    Single sample augment
    Args:
        snr (float): all pixels equal or above this value are inverted.
        p (float): probability of the image being color inverted. Default value is 0.5
    """

    def __init__(self, snr=20., p=0.5) -> None:
        super().__init__()
        self.snr = snr
        self.p = p

    @torch.no_grad()
    def forward(self, x):
        '''
        Returns: noise + x
        '''
        b, w = x.shape[0], x.shape[-1]
        random_seed = (torch.rand(list(x.shape[:-1]) + [1], device=x.device) < self.p).type(x.dtype)

        Ps = torch.sum(x ** 2, dim=0) / w
        Pn = Ps / (10 ** (self.snr / 10))
        noise = torch.randn_like(x, device=x.device) * torch.sqrt(Pn).expand_as(x)
        return x + random_seed * noise


class HShift(torch.nn.Module):
    """
    Single sample augment
    Args:
        p (float): probability of the image being color inverted. Default value is 0.5
    """

    def __init__(self, p=0.5) -> None:
        super().__init__()
        self.p = p

    @torch.no_grad()
    def forward(self, x) -> Tensor:
        b, w = x.shape[0], x.shape[-1]
        rolls = []
        random_seed = torch.rand(b) < self.p
        for index in range(b):
            rolls.append(torch.roll(x[index], random.randint(0, w), dims=-1) if random_seed[index] else x[index])
        return torch.stack(rolls)


class VShift(torch.nn.Module):
    """
    Single sample augment
    Args:
        p (float): probability of the image being color inverted. Default value is 0.5
    """

    def __init__(self, p=0.5) -> None:
        super().__init__()
        self.p = p

    @torch.no_grad()
    def forward(self, x):
        _mean = torch.mean(x, dim=-1).unsqueeze(dim=-1)
        _max, _ = torch.max(torch.abs(x - _mean), dim=-1)

        random_seed = (torch.rand_like(_max, device=_max.device) < self.p).float()
        value_float = (torch.randn_like(_max, device=_max.device) * (_max / 2) * random_seed).unsqueeze(
            dim=-1).expand_as(
            x)
        return x + value_float


class VFlip(torch.nn.Module):
    """
    Single sample augment
    Args:
        snr (float): all pixels equal or above this value are inverted.
        p (float): probability of the image being color inverted. Default value is 0.5
    """

    def __init__(self, p=0.5) -> None:
        super().__init__()
        self.p = p

    @torch.no_grad()
    def forward(self, x: Tensor) -> Tensor:
        """
        Args:
            x:
        Returns:
            PIL Image or Tensor: Randomly solarized image.
        """
        b = x.shape[0]
        random_seed = torch.rand(b) < self.p
        flip = []
        for index in range(b):
            flip.append(x[index].neg() if random_seed[index] else x[index])
        return torch.stack(flip)


class HFlip(torch.nn.Module):
    """
    Single sample augment
    Args:
        snr (float): all pixels equal or above this value are inverted.
        p (float): probability of the image being color inverted. Default value is 0.5
    """

    def __init__(self, p=0.5) -> None:
        super().__init__()
        self.p = p

    @torch.no_grad()
    def forward(self, x):
        """
        Args:
            img (PIL Image or Tensor): Image to be solarized.

        Returns:
            PIL Image or Tensor: Randomly solarized image.
        """
        b = x.shape[0]
        random_seed = torch.rand(b) < self.p
        flip = []
        for index in range(b):
            flip.append(x[index].flip(-1) if random_seed[index] else x[index])
        return torch.stack(flip)


@DeprecationWarning
def _get_gaussian_kernel1d(kernel_size: int, sigma: float) -> Tensor:
    ksize_half = (kernel_size - 1) * 0.5

    x = torch.linspace(-ksize_half, ksize_half, steps=kernel_size)
    pdf = torch.exp(-0.5 * (x / sigma).pow(2))
    kernel1d = pdf / pdf.sum()

    return kernel1d


@DeprecationWarning
class Rotate(torch.nn.Module):
    """
    Single sample augment
    Args:
        snr (float): all pixels equal or above this value are inverted.
        p (float): probability of the image being color inverted. Default value is 0.5
        random rotate between +15 and -15 angle
    """

    def __init__(self, p=0.5, max_angle=5, rotate_loc=None) -> None:
        super().__init__()
        self.p = p
        self.x = None
        self.max_angle = max_angle
        self.rotate_loc = rotate_loc

    @torch.no_grad()
    def forward(self, y):
        """
        Args:
            img (PIL Image or Tensor): Image to be solarized.

        Returns:
            PIL Image or Tensor: Randomly solarized image.
        """
        if random.uniform(0, 1) < self.p:
            # if self.rotate_loc is None:
            # self.rotate_loc = torch.randint(0, y.shape[-1], (1,)).item()
            rotate_loc = random.randint(0, y.shape[-1])
            if self.x is None:
                self.x = torch.arange(0 - rotate_loc, y.shape[-1] - rotate_loc, device=y.device).expand_as(y)
            random_angle = (torch.pi * 2 / 360 * self.max_angle * torch.rand(1, device=y.device)).expand_as(y)
            return self.x * torch.sin(random_angle) + y * torch.cos(random_angle)
        else:
            return y


@DeprecationWarning
class GaussianBlur(torch.nn.Module):
    """Blurs image with randomly chosen Gaussian blur.
    If the image is torch Tensor, it is expected
    to have [..., C, H, W] shape, where ... means an arbitrary number of leading dimensions.

    Args:
        kernel_size (int or sequence): Size of the Gaussian kernel.
        sigma (float or tuple of float (min, max)): Standard deviation to be used for
            creating kernel to perform blurring. If float, sigma is fixed. If it is tuple
            of float (min, max), sigma is chosen uniformly at random to lie in the
            given range.

    Returns:
        PIL Image or Tensor: Gaussian blurred version of the input image.

    """

    def __init__(self, kernel_size, p, sigma=(0.1, 2.0)) -> None:
        super().__init__()
        self.p = p
        self.kernel_size = kernel_size
        if self.kernel_size <= 0 or self.kernel_size % 2 == 0:
            raise ValueError("Kernel size value should be an odd and positive number.")

        if isinstance(sigma, numbers.Number):
            if sigma <= 0:
                raise ValueError("If sigma is a single number, it must be positive.")
            sigma = (sigma, sigma)
        elif isinstance(sigma, Sequence) and len(sigma) == 2:
            if not 0.0 < sigma[0] <= sigma[1]:
                raise ValueError("sigma values should be positive and of the form (min, max).")
        else:
            raise ValueError("sigma should be a single number or a list/tuple with length 2.")

        self.sigma = sigma

    @staticmethod
    def get_params(sigma_min: float, sigma_max: float) -> float:
        """Choose sigma for random gaussian blurring.

        Args:
            sigma_min (float): Minimum standard deviation that can be chosen for blurring kernel.
            sigma_max (float): Maximum standard deviation that can be chosen for blurring kernel.

        Returns:
            float: Standard deviation to be passed to calculate kernel for gaussian blurring.
        """
        return torch.empty(1).uniform_(sigma_min, sigma_max).item()

    @torch.no_grad()
    def forward(self, img: Tensor) -> Tensor:
        """
        Args:
            img (PIL Image or Tensor): image to be blurred.

        Returns:
            PIL Image or Tensor: Gaussian blurred image
        """
        if random.uniform(0, 1) <= self.p:
            sigma = self.get_params(self.sigma[0], self.sigma[1])
            kernel = _get_gaussian_kernel1d(self.kernel_size, sigma)
            kernel = kernel.expand(1, 1, 1, self.kernel_size).to(img.device)
            padding = [self.kernel_size // 2, self.kernel_size // 2, 0, 0]
            img = torch_pad(img, padding, mode="reflect")
            img = conv2d(img, kernel, groups=img.shape[-3])
            return img
        else:
            return img

    def __repr__(self) -> str:
        s = f"{self.__class__.__name__}(kernel_size={self.kernel_size}, sigma={self.sigma})"
        return s


# @DeprecationWarning
def resize(input,
           size=None,
           scale_factor=None,
           mode='nearest',
           align_corners=None,
           warning=True):
    if warning:
        if size is not None and align_corners:
            input_h, input_w = tuple(int(x) for x in input.shape[2:])
            output_h, output_w = tuple(int(x) for x in size)
            if output_h > input_h or output_w > output_h:
                if ((output_h > 1 and output_w > 1 and input_h > 1
                     and input_w > 1) and (output_h - 1) % (input_h - 1)
                        and (output_w - 1) % (input_w - 1)):
                    warnings.warn(
                        f'When align_corners={align_corners}, '
                        'the output would more aligned if '
                        f'input size {(input_h, input_w)} is `x+1` and '
                        f'out size {(output_h, output_w)} is `nx+1`')
    return F.interpolate(input, size, scale_factor, mode, align_corners)


class TimeMasking(torch.nn.Module):
    def __init__(self, block_size, ratio, flip_p, shift_p, norm) -> None:
        super(TimeMasking, self).__init__()

        self.block_size = block_size
        self.ratio = ratio

        self.transform = Compose(
            [
                VFlip(flip_p),
                HFlip(flip_p),
                HShift(shift_p)
            ]
        )
        if norm: self.transform.transforms.append(Normalization())

    @torch.no_grad()
    def forward(self, x: Tensor) -> Tensor:
        x = self.transform(x.clone())
        B, W = x.shape[0], x.shape[-1]
        mshape = B, 1, round(W / self.block_size)
        time_mask = torch.rand(mshape, device=x.device)
        time_mask = (time_mask > self.ratio).float()
        time_mask = resize(time_mask, size=W)
        return x * time_mask.view_as(x)


@DeprecationWarning
class Resample(torch.nn.Module):
    def __init__(
            self,
            orig_freq: int = 12000,
            resampling_method: str = "sinc_interpolation",
            lowpass_filter_width: int = 6,
            rolloff: float = 0.99,
    ) -> None:
        super().__init__()
        self.orig_freq = orig_freq
        self.resample_method = resampling_method
        self.lowpass_filter_width = lowpass_filter_width
        self.rolloff = rolloff

    def forward(self, x: Tensor) -> Tensor:
        b = x.shape[0]
        resampled_x = []
        # 512 < resample_ratio / self.orig_freq * 1024 < 1536 ->
        random_seed = torch.rand(b, device=x.device) * 0.4 + 0.8
        for index in range(b):
            resample_x_index = T.Resample(self.orig_freq, (self.orig_freq * random_seed[index]).int())(x[index])
            resampled_x.append(resample_x_index if random_seed[index] >= 1. else torch.cat(
                (resample_x_index, resample_x_index.flip(-1)), dim=-1))
        return torch.stack(resampled_x)


def random_select(*perturbed_samples_set):
    num_perturbed = len(perturbed_samples_set)
    for i in range(1, num_perturbed):
        assert perturbed_samples_set[i - 1].shape == perturbed_samples_set[i].shape
    batch_size = perturbed_samples_set[0].shape[0]
    random_selected = []
    random_seeds = torch.randint(0, num_perturbed, (batch_size,))
    for i in range(batch_size):
        random_selected.append(perturbed_samples_set[random_seeds[i]][i].unsqueeze(0))
    return torch.cat(random_selected, dim=0)



__all__ = ['Pu']

import os
import random
import numpy as np
from scipy.io import loadmat


class Pu_Utils(object):
    bearing_condition = {
        'K001': {'Damage_Component': 'NO', 'Damage_Level': 'NO', 'Damage_Method': 'NO'},
        'K002': {'Damage_Component': 'NO', 'Damage_Level': 'NO', 'Damage_Method': 'NO'},
        'K003': {'Damage_Component': 'NO', 'Damage_Level': 'NO', 'Damage_Method': 'NO'},
        'K004': {'Damage_Component': 'NO', 'Damage_Level': 'NO', 'Damage_Method': 'NO'},
        'K005': {'Damage_Component': 'NO', 'Damage_Level': 'NO', 'Damage_Method': 'NO'},
        'K006': {'Damage_Component': 'NO', 'Damage_Level': 'NO', 'Damage_Method': 'NO'},
        'KA01': {'Damage_Component': 'OR', 'Damage_Level': '1', 'Damage_Method': 'EDM'},
        'KA03': {'Damage_Component': 'OR', 'Damage_Level': '2', 'Damage_Method': 'electric engraver'},
        'KA05': {'Damage_Component': 'OR', 'Damage_Level': '1', 'Damage_Method': 'electric engraver'},
        'KA06': {'Damage_Component': 'OR', 'Damage_Level': '2', 'Damage_Method': 'electric engraver'},
        'KA07': {'Damage_Component': 'OR', 'Damage_Level': '1', 'Damage_Method': 'drilling'},
        'KA08': {'Damage_Component': 'OR', 'Damage_Level': '2', 'Damage_Method': 'drilling'},
        'KA09': {'Damage_Component': 'OR', 'Damage_Level': '2', 'Damage_Method': 'drilling'},
        'KI01': {'Damage_Component': 'IR', 'Damage_Level': '1', 'Damage_Method': 'EDM'},
        'KI03': {'Damage_Component': 'IR', 'Damage_Level': '1', 'Damage_Method': 'electric engraver'},
        'KI05': {'Damage_Component': 'IR', 'Damage_Level': '1', 'Damage_Method': 'electric engraver'},
        'KI07': {'Damage_Component': 'IR', 'Damage_Level': '2', 'Damage_Method': 'electric engraver'},
        'KI08': {'Damage_Component': 'IR', 'Damage_Level': '2', 'Damage_Method': 'electric engraver'},
    }

    def __init__(self):
        super(Pu_Utils, self).__init__()

    @staticmethod
    def _get_file_labels(root, work_state):
        def get_label(value):
            Label = {
                'NO_NO': 0,
                'OR_1': 1,
                'OR_2': 2,
                'IR_1': 3,
                'IR_2': 4,
            }
            key = value['Damage_Component'] + '_' + value['Damage_Level']
            return Label[key]

        file_labels = []
        for key in Pu_Utils.bearing_condition:
            path, label = os.path.join(root, key, f'{work_state}_{key}_1.mat'), get_label(
                Pu_Utils.bearing_condition[key])
            file_labels.append((path, label))
        return file_labels

    @staticmethod
    def _get_data(file, label, length, hop_size):
        fl = loadmat(file)[file.split(os.sep)[-1].split('.')[0]]
        fl = fl[0][0][2][0][6][2]  # Take out the data
        fl = fl.reshape(-1)
        samples = []
        start_index = 0
        while (start_index + length < fl.size):
            samples.append(fl[start_index: start_index + length])
            start_index += hop_size
        return np.vstack(samples).astype(np.float32), np.full(len(samples), label, dtype=np.longlong)

    @staticmethod
    def get_sample_labels(root, work_state, length, hop_size):
        file_labels = Pu_Utils._get_file_labels(root, work_state)
        samples, labels = [], []
        for file, label in file_labels:
            sample, label = Pu_Utils._get_data(file, label, length, hop_size)
            samples.append(sample)
            labels.append(label)
        return np.concatenate(samples, axis=0), np.hstack(labels)


class Pu:
    """Pu Dataset.
      dataset link: https://www.aliyundrive.com/s/AanRmmBNZna  extract code: 4qq9
      Args:
          root (str): Root directory of dataset
          work_state (str): experiment work_state of dataset
          length (int): window length (or num of vibrations of each input)
          hop_size (int): hop length (or num of vibrations of each input
          shuffle (bool): whether shuffle dataset
      """

    work_state = ["N15_M07_F10", "N09_M07_F10", "N15_M01_F10", "N15_M07_F04"]
    CLASSES = [
        'Normal',
        'OR-1',
        'OR-2',
        'IR-1',
        'IR-2',
    ]

    def __init__(self, root: str, work_state: str, length=1024, hop_size=1024, shuffle=False):
        assert work_state in self.work_state
        super(Pu, self).__init__()

        self.x, self.y = Pu_Utils.get_sample_labels(root, work_state, length, hop_size)
        if shuffle:
            self._shuffle()
        self.classes = Pu.CLASSES
        self.num_classes = len(self.classes)

    def _shuffle(self) -> None:
        # shuffle training samples
        index = list(range(self.x.shape[0]))
        random.Random(0).shuffle(index)
        self.x = self.x[index]
        self.y = self.y[index]


#!/usr/bin/env python
# coding: utf-8

import setuptools

# read the contents of your README file
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
setuptools.setup(
    name='bearing-python',
    version='1.0.4',
    author='Xu Fangzheng',
    author_email='fonderxu@163.com',
    url='https://github.com/fonderxu/Bearing-fault-diagnosis-datasets',
    description=u'README.md',
    install_requires=[],
    entry_points={
        'console_scripts': []
    },
    readme="README.md",
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_namespace_packages(include=["bearing", "bearing.*"], ),
    include_package_data=True
)




import os
import errno
import random
import urllib.request as request
import numpy as np
from scipy.io import loadmat


class Cwru:
    """CWRU Dataset.

      dataset link:  https://pan.xunlei.com/s/VNNVs4cGhuL1NqGrauTU19zYA1?pwd=kta7# extract code: kta7
      Args:
          root (str): Root directory of dataset
          exp (str): experiment state of dataset
          rpm (str): The task (domain) to create dataset. Choices include ``'600'``, \
              ``'800'`` and ``'1000'``.
          length (int): window length (or num of vibrations of each input)
          hop_size (int): hop length (or num of vibrations of each input
          shuffle (bool): whether shuffle dataset
      """

    sample_location = [
        "12DriveEndFault",
        "12FanEndFault",
        "48DriveEndFault"
    ]

    domain_list = [
        "1797",
        "1772",
        "1750",
        "1730",
    ]

    CLASSES = {
        '12DriveEndFault_1797': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', '0.028-Ball', '0.028-InnerRace', 'Normal'),
        '12DriveEndFault_1772': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', '0.028-Ball', '0.028-InnerRace', 'Normal'),
        '12DriveEndFault_1750': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', '0.028-Ball', '0.028-InnerRace', 'Normal'),
        '12DriveEndFault_1730': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', '0.028-Ball', '0.028-InnerRace', 'Normal'),
        '12FanEndFault_1797': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace3', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace',
            '0.021-OuterRace6',
            'Normal'),
        '12FanEndFault_1772': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace3', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace3', 'Normal'),
        '12FanEndFault_1750': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace3', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace3', 'Normal'),
        '12FanEndFault_1730': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace3', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace3', 'Normal'),
        '48DriveEndFault_1797': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', 'Normal'),
        '48DriveEndFault_1772': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', 'Normal'),
        '48DriveEndFault_1750': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', 'Normal'),
        '48DriveEndFault_1730': (
            '0.007-Ball', '0.007-InnerRace', '0.007-OuterRace12', '0.007-OuterRace3', '0.007-OuterRace6', '0.014-Ball',
            '0.014-InnerRace', '0.014-OuterRace6', '0.021-Ball', '0.021-InnerRace', '0.021-OuterRace12',
            '0.021-OuterRace3',
            '0.021-OuterRace6', 'Normal'),
    }

    def __init__(self, root: str, exp: str, rpm: str, length=1024, hop_size=1024, shuffle=False) -> None:
        assert exp in self.sample_location and rpm in self.domain_list
        super(Cwru, self).__init__()
        # root directory of all data
        rdir = os.path.join(root, exp, rpm)
        fmeta = os.path.join(os.path.dirname(__file__), 'metadata.txt')
        all_lines = open(fmeta).readlines()
        lines = []
        for line in all_lines:
            l = line.split()
            if (l[0] == exp or l[0] == 'NormalBaseline') and l[1] == rpm:
                lines.append(l)

        self.length = length  # sequence length
        self.hop_size = hop_size
        self._load_and_slice_data(rdir, lines)
        # shuffle training and test arrays
        if shuffle:
            self._shuffle()
        self.labels = self.CLASSES[f'{exp}_{rpm}']
        self.nclasses = len(self.labels)  # number of classes

    def _mkdir(self, path) -> None:
        try:
            os.makedirs(path)
        except OSError as exc:
            if exc.errno == errno.EEXIST and os.path.isdir(path):
                pass
            else:
                print("can't create directory '{}''".format(path))
                exit(1)

    def _download(self, fpath, link):
        print("Downloading to: '{}'".format(fpath))
        request.urlretrieve(link, fpath)

    def _load_and_slice_data(self, rdir, infos) -> None:
        self.x = np.zeros((0, self.length))
        self.y = np.zeros(0, dtype=np.longlong)
        for idx, info in enumerate(infos):
            # directory of this file
            fdir = os.path.join(rdir, info[0], info[1])
            self._mkdir(fdir)
            fpath = os.path.join(fdir, info[2] + '.mat')
            if not os.path.exists(fpath):
                self._download(fpath, info[3].rstrip('\n'))

            x, y = self._get_file_data(fpath, idx, self.length, self.hop_size)
            self.x = np.vstack((self.x, x))
            self.y = np.hstack((self.y, y))

    def _get_file_data(self, file, label, length, hop_size) -> tuple[np.ndarray, np.ndarray]:
        mat_dict = loadmat(file)
        key = list(filter(lambda x: 'DE_time' in x, mat_dict.keys()))[0]
        data = mat_dict[key][:, 0]

        samples = []
        start_index = 0
        while(start_index + length < data.size):
            samples.append(data[start_index: start_index + length])
            start_index += hop_size
        return np.vstack(samples).astype(np.float32), np.full(len(samples), label, dtype=np.longlong)

    def _shuffle(self) -> None:
        # shuffle training samples
        index = list(range(self.x.shape[0]))
        random.Random(0).shuffle(index)
        self.x = self.x[index]
        self.y = self.y[index]



